"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const emitter = require("ts-emitter");
const ts = require("typescript");
class Collection {
    constructor(nodes, root) {
        this.root = root;
        this._nodes = nodes;
    }
    static createRootCollection(root) {
        return new Collection([root], root);
    }
    find(kind) {
        const marked = [];
        const visitor = (node) => {
            if (node.kind === kind) {
                marked.push(node);
            }
            else {
                ts.forEachChild(node, visitor);
            }
        };
        this.nodes.forEach(node => {
            ts.forEachChild(node, visitor);
        });
        return new Collection(marked, this.root);
    }
    replaceWith(fn) {
        const replacer = (context) => (rootNode) => {
            const visitor = (node) => {
                const markedNode = this.nodes.find(item => item === node);
                if (markedNode) {
                    const replaced = fn(markedNode);
                    replaced.original = markedNode;
                    if (replaced.text && replaced.text !== markedNode.text) {
                        replaced.newText = replaced.text;
                    }
                    return replaced;
                }
                return ts.visitEachChild(node, visitor, context);
            };
            return ts.visitNode(rootNode, visitor);
        };
        this.root = ts.transform(this.root, [replacer]).transformed[0];
        return this;
    }
    get nodes() {
        return this._nodes;
    }
    toSource() {
        return emitter.toSource(this.root);
    }
}
exports.Collection = Collection;
